@@ -3,6 +3,7 @@ from .decorators import *
 import tuned.logs
 
 import os
+import errno
 import struct
 import glob
 from tuned.utils.commands import commands
@@ -12,7 +13,25 @@ cmd = commands()
 
 class VMPlugin(base.Plugin):
 	"""
-	Plugin for tuning memory management.
+	`vm`::
+	
+	Enables or disables transparent huge pages depending on value of the
+	[option]`transparent_hugepages` option. The option can have one of three
+	possible values `always`, `madvise` and `never`.
+	+
+	.Disable transparent hugepages
+	====
+	----
+	[vm]
+	transparent_hugepages=never
+	----
+	====
+	+
+	The [option]`transparent_hugepage.defrag` option specifies the
+	defragmentation policy. Possible values for this option are `always`,
+	`defer`, `defer+madvise`, `madvise` and `never`. For a detailed
+	explanation of these values refer to
+	link:https://www.kernel.org/doc/Documentation/vm/transhuge.txt[Transparent Hugepage Support].
 	"""
 
 	@classmethod
@@ -39,7 +58,7 @@ class VMPlugin(base.Plugin):
 		return path
 
 	@command_set("transparent_hugepages")
-	def _set_transparent_hugepages(self, value, sim):
+	def _set_transparent_hugepages(self, value, sim, remove):
 		if value not in ["always", "never", "madvise"]:
 			if not sim:
 				log.warn("Incorrect 'transparent_hugepages' value '%s'." % str(value))
@@ -48,13 +67,14 @@ class VMPlugin(base.Plugin):
 		cmdline = cmd.read_file("/proc/cmdline", no_error = True)
 		if cmdline.find("transparent_hugepage=") > 0:
 			if not sim:
-				log.info("transparent_hugepage is already set in kernel boot cmdline, ingoring value from profile")
+				log.info("transparent_hugepage is already set in kernel boot cmdline, ignoring value from profile")
 			return None
 
 		sys_file = os.path.join(self._thp_path(), "enabled")
 		if os.path.exists(sys_file):
 			if not sim:
-				cmd.write_to_file(sys_file, value)
+				cmd.write_to_file(sys_file, value, \
+					no_error = [errno.ENOENT] if remove else False)
 			return value
 		else:
 			if not sim:
@@ -63,8 +83,8 @@ class VMPlugin(base.Plugin):
 
         # just an alias to transparent_hugepages
 	@command_set("transparent_hugepage")
-	def _set_transparent_hugepage(self, value, sim):
-		self._set_transparent_hugepages(value, sim)
+	def _set_transparent_hugepage(self, value, sim, remove):
+		self._set_transparent_hugepages(value, sim, remove)
 
 	@command_get("transparent_hugepages")
 	def _get_transparent_hugepages(self):
@@ -80,11 +100,12 @@ class VMPlugin(base.Plugin):
 		return self._get_transparent_hugepages()
 
 	@command_set("transparent_hugepage.defrag")
-	def _set_transparent_hugepage_defrag(self, value, sim):
+	def _set_transparent_hugepage_defrag(self, value, sim, remove):
 		sys_file = os.path.join(self._thp_path(), "defrag")
 		if os.path.exists(sys_file):
 			if not sim:
-				cmd.write_to_file(sys_file, value)
+				cmd.write_to_file(sys_file, value, \
+					no_error = [errno.ENOENT] if remove else False)
 			return value
 		else:
 			if not sim:
